# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################
import time

from osv import fields, osv
from tools.translate import _
import decimal_precision as dp

class account_invoice_pay(osv.osv_memory):
    """
    Generate pay invoice wizard, user can make partial or full payment for invoice.
    """
    _name = "account.invoice.pay"
    _description = "Pay Invoice  "
    _columns = {
        'amount': fields.float('Amount paid', required=True, digits_compute=dp.get_precision('Account')),
        'name': fields.char('Entry Name', size=64, required=True),
        'date': fields.date('Date payment', required=True),
        'journal_id': fields.many2one('account.journal', 'Journal/Payment Mode', required=True, domain=[('type', '=', 'cash')]),
        'period_id': fields.many2one('account.period', 'Period', required=True),
        }

    def view_init(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        invoice = self.pool.get('account.invoice').browse(cr, uid, context.get('active_id', []), context=context)
        if invoice and invoice.state in ['draft', 'proforma2', 'cancel']:
            raise osv.except_osv(_('Error !'), _('Can not pay draft/proforma/cancel invoice.'))

    def _get_period(self, cr, uid, context=None):
        """
        Initialise Period
        """
        ids = self.pool.get('account.period').find(cr, uid, context=context)
        period_id = False
        if ids:
            period_id = ids[0]
        return period_id

    def _get_amount(self, cr, uid, context=None):
        """
        Get default value of Amount paid
        """
        if context is None:
            context = {}
        invoice = self.pool.get('account.invoice').browse(cr, uid, context.get('active_id'), context=context) 
        return invoice and invoice.residual

    _defaults = {
        'date': lambda *a: time.strftime('%Y-%m-%d'),
        'period_id': _get_period,
        'amount': _get_amount,
        }
    
account_invoice_pay()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: