# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

from osv import osv, fields
from tools.translate import _

class print_check(osv.osv_memory):
    """
    Check Printing
    """
    _name = "print.check"
    _description = "Print Check"
    _columns = {
        'name': fields.char('Next Check Number', size=32, help='Next check number'),
        'nxt_seq': fields.char('Next Check Number', size=32, help='Next check number'),
        'new_no': fields.integer('Update Check Number', help= 'Enter new check number here if you wish to update'),
        'preprint_msg': fields.text('Message'),
        'status': fields.selection([('voided', 'Voided'), ('stop_pay', 'Stop Pay Placed'), ('lost', 'Lost'), ('unk', 'Unknown')], 'Status'),
        'pre_printed': fields.boolean('Pre Printed'),
        'state':fields.selection([
            ('print', 'print'),
            ('printed_reprint', 'printed_reprint'),
            ('printed', 'printed'),
            ('reprint_new', 'reprint_new'),
            ('reprint', 'reprint'),
            ('update_check_no', 'update_check_no'),
            ('do_update', 'do_update'),
            ('do_action', 'do_action'),
            ('top', 'top'),
            ('middle', 'middle'),
            ('bottom', 'bottom')], 'State'),
        'print_new': fields.boolean('Print a New Check'),
        'reprint': fields.boolean('Reprint existing check'),
        'update_check_no': fields.boolean('Update Supplier Payment’s Check No'),
        }

    def get_id(self, cr, uid, sequence_id, test='id', context=None):
        """
        Function to find next sequence number
        """
        seq_pool = self.pool.get('ir.sequence')
        assert test in ('code', 'id')
        company_id = self.pool.get('res.users').read(cr, uid, uid, ['company_id'], context=context)['company_id'][0] or None
        cr.execute('''SELECT id, number_next, prefix, suffix, padding
                      FROM ir_sequence
                      WHERE %s=%%s
                      AND active=true
                      AND (company_id = %%s or company_id is NULL)
                      ORDER BY company_id, id
                      FOR UPDATE NOWAIT''' % test,
                      (sequence_id, company_id))
        res = cr.dictfetchone()
        if res:
            if res['number_next']:
                return (seq_pool._process(res['prefix']) + '%%0%sd' % res['padding'] % res['number_next'] + seq_pool._process(res['suffix']),
                        res['number_next'])
            else:
                return (seq_pool._process(res['prefix']) + seq_pool._process(res['suffix']), 0)
        return False

    def _get_nxt_no(self, cr, uid, context=None):
        """
        Function to find next sequence number and update sequence
        """
        if context is None:
            context = {}
        if 'active_ids' in context:
            voucher_id = self.pool.get('account.voucher').browse(cr, uid, context['active_ids'], context=context)[0]
            if not voucher_id.journal_id.check_sequence:
                raise osv.except_osv(_('Warning!'), _('Please add "Check Sequence" for journal %s.'%str(voucher_id.journal_id.name)))
            res = self.get_id(cr, uid, voucher_id.journal_id.check_sequence.id, test='id', context=context)
            return res and res[0]
        else:
            return False

    def _get_new_no(self, cr, uid, context=None):
        """
        Function to get the next number used to generate sequence
        """
        if context is None:
            context = {}
        res = []
        if 'active_ids' in context:
            voucher_id = self.pool.get('account.voucher').browse(cr, uid, context['active_ids'][0], context=context)
            res = self.get_id(cr, uid, voucher_id.journal_id.check_sequence.id , test='id', context=context)
        return res and res[1]

    def _get_state(self, cr, uid, context=None):
        """
        Function to initialise state
        """
        if context is None:
            context = {}
        state = 'print'
        if not (context.get('active_model') == 'account.voucher' and context.get('active_ids')):
            raise osv.except_osv(_('Warning!'), _('Wrong model or unable to find active ids'))
        for voucher_id in self.pool.get('account.voucher').browse(cr, uid, context['active_ids'], context=context):
            if voucher_id.chk_seq and voucher_id.journal_id.use_preprint_check:
                state = 'printed'
            elif voucher_id.chk_seq and state != 'printed':
                state = 'printed_reprint'
            if voucher_id.state != 'posted':
                raise osv.except_osv(_('Warning!'), _('Payment is not posted. Please Validate Payment First!'))
            if not voucher_id.journal_id.check_sequence:
                raise osv.except_osv(_('Warning!'), _('Please add "Check Sequence" for journal %s.'%str(voucher_id.journal_id.name)))
        return state

    def _get_pre_printed(self, cr, uid, context=None):
        """
        Function to check whether the check is pre printed or not
        """
        if context is None:
            context = {}
        if not (context.get('active_model') == 'account.voucher' and context.get('active_ids')):
            raise osv.except_osv(_('Warning!'), _('Wrong model or unable to find active ids'))
        for voucher_id in self.pool.get('account.voucher').browse(cr, uid, context['active_ids'], context=context):
            return voucher_id.journal_id.use_preprint_check
        return False

    def _get_msg(self, cr, uid, context=None):
        """
        Function to initialize preprint_msg
        """
        if context is None:
            context = {}

        msg1 = 'This Payment has already been paid with check:\n'
        msg2 = 'These Payments have already been paid with checks:\n'
        msg3 = 'Some of these Payments have already been paid with checks:\n'

        chk_nos = []
        voucher_ids = self.pool.get('account.voucher').browse(cr, uid, context.get('active_ids', []), context=context)
        for voucher in voucher_ids:
            if voucher.chk_seq:
                chk_nos.append(str(voucher.chk_seq))
        msg = ''
        if len(chk_nos) == 1:
            msg = msg1 + str(chk_nos[0])
        elif len(chk_nos) == len(context.get('active_ids')):
            msg = msg2 + '\n'.join(chk_nos)
        else:
            msg = msg3 + '\n'.join(chk_nos)
        return msg

    _defaults = {
        'name': 'Check sequence',
        'nxt_seq': _get_nxt_no,
        'new_no': _get_new_no,
        'preprint_msg': _get_msg,
        'pre_printed': _get_pre_printed,
        'state': _get_state
        }

    def check_option(self, cr, uid, ids, context=None):
        """
        Function to check the option if check is already printed
        """
        if context is None:
            context = {}
        data = self.browse(cr, uid, ids[0], context=context)
        if data.print_new:
            msg =  'What happened to the existing check no ' + str(data.preprint_msg.split(':\n')[1]).replace('\n', ', ') + '?'
            self.write(cr, uid, ids, {'preprint_msg': msg, 'state': 'reprint_new'}, context=context)
        elif data.reprint:
            company_obj = self.pool.get('res.users').browse(cr, uid, uid, context=context).company_id
            if company_obj.check_layout == 'top':
                report_name = 'account.print.check.top'
            if company_obj.check_layout == 'middle':
                report_name = 'account.print.check.middle'
            if company_obj.check_layout == 'bottom':
                report_name = 'account.print.check.bottom'

            return {
            'type': 'ir.actions.report.xml',
            'report_name': report_name,
            'datas': {
                'model': 'account.voucher',
                'id': context.get('active_ids', False) and context['active_ids'][0],
                'ids': context.get('active_ids', []),
                'report_type': 'pdf'
                },
            'nodestroy': False
            }
        elif data.update_check_no:
            msg =  'What happened to the existing check no ' + str(data.preprint_msg.split(':\n')[1]).replace('\n', ', ') + '?'
            self.write(cr, uid, ids, {'preprint_msg': msg, 'state': 'update_check_no'}, context=context)
        return {'nodestroy': True}

    def get_nxt_seq(self, cr, uid, check_sequence_id, test='id', context=None):
        """
        Function to find the next check number without conflict
        """
        next_seq = self.pool.get('ir.sequence').get_id(cr, uid, voucher.journal_id.check_sequence.id, test='id', context=context)
        while(self.pool.get('check.log').search(cr, uid, [('check_no', '=', next_seq)], context=context)):
            next_seq = self.pool.get('ir.sequence').get_id(cr, uid, voucher.journal_id.check_sequence.id, test='id', context=context)
        return next_seq

    def print_check(self, cr, uid, ids, context=None):
        """
        Function to print check
        """
        if context is None:
            context = {}

        if not context.get('active_ids'):
            return []

        seq = {}
        voucher_objs = self.pool.get('account.voucher').browse(cr, uid, context['active_ids'], context=context)
        data = self.browse(cr, uid, ids[0], context=context)
        #FIXME: Please check whether we need 3 loops here.
        for voucher in voucher_objs:
            if voucher.journal_id.check_sequence:
                seq[voucher.journal_id.check_sequence.id] = True
            else:
                raise wizard.except_wizard(_('Warning'), _('Please add "Check Sequence" for journal %s.'%str(voucher.journal_id.name)))
        for seq_id in seq:
            #FIXME: Please check whether this call is necessary
            nxt_no = self.pool.get('ir.sequence').read(cr, uid, seq_id, ['number_next'], context=context)['number_next']
            self.pool.get('ir.sequence').write(cr, uid, [seq_id], {'number_next': data.new_no}, context=context)

        for voucher in voucher_objs:
            next_seq = self.pool.get('ir.sequence').get_id(cr, uid, voucher.journal_id.check_sequence.id, test='id', context=context)
            self.pool.get('account.voucher').write(cr, uid, [voucher.id],{'chk_seq': next_seq, 'chk_status': True}, context=context)
            self.pool.get('check.log').create(cr, uid, {'name': voucher.id, 'status': 'active', 'check_no': next_seq}, context=context)

        if data.state == 'print':
            company_obj = self.pool.get('res.users').browse(cr, uid, uid, context=context).company_id
            if company_obj.check_layout == 'top':
                report_name = 'account.print.check.top'
            if company_obj.check_layout == 'middle':
                report_name = 'account.print.check.middle'
            if company_obj.check_layout == 'bottom':
                report_name = 'account.print.check.bottom'
            return {
                'type': 'ir.actions.report.xml',
                'report_name': report_name,
                'datas': {
                    'model': 'account.voucher',
                    'id': context.get('active_ids', False) and context.get('active_ids')[0],
                    'ids': context.get('active_ids', []),
                    'report_type': 'pdf'
                    },
                'nodestroy': False
                }
        else:
            return {}

    def onchange_chkbx(self, cr, uid, ids, value, field, context=None):
        """
        Function to update check box print_new, reprint and update_check_no
        """
        ret = {
            'print_new': False,
            'reprint': False,
            'update_check_no': False,
            }
        if value:
            ret[field] = True
        return {'value': ret}

    def _update_check_log_status(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        data = self.browse(cr, uid, ids[0], context=context)
        voucher_ids = self.pool.get('account.voucher').browse(cr, uid, context.get('active_ids', []), context=context)
        check_log_obj = self.pool.get('check.log')
        if data.status:
            for voucher in voucher_ids:
                if voucher.chk_seq:
                    chk_log_ids = check_log_obj.search(cr, uid, [('check_no', '=', voucher.chk_seq), ('status', '=', 'active')], context=context)
                    check_log_obj.write(cr, uid, chk_log_ids, {'status': data.status or 'unk'}, context=context)

    def update_no(self, cr, uid, ids, context=None):
        """
        Function to update check log status
        """
        self._update_check_log_status(cr, uid, ids, context=context)
        self.write(cr, uid, ids, {'preprint_msg': '', 'state': 'do_update'}, context=context)

    def print_new(self, cr, uid, ids, context=None):
        """
        Function to update check log status
        """
        self._update_check_log_status(cr, uid, ids, context=context)
        self.write(cr, uid, ids,{'preprint_msg': '', 'state': 'print'}, context=context)

print_check()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
