# -*- ecoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

from osv import fields, osv

class bom(osv.osv):
    
    def _update_product_weight(self, cr, uid, bom_obj, context=None):
        weight = 0.00
        weight_net = 0.00
        standard_price = 0.00
        child_ids = self.search(cr, uid, [('bom_id', 'child_of', [bom_obj.id])], context=context)
        if child_ids and bom_obj.id in child_ids:
            child_ids.remove(bom_obj.id)
        if len(child_ids) == 0:
            weight = bom_obj.product_id.weight * bom_obj.product_qty
            weight_net = bom_obj.product_id.weight_net * bom_obj.product_qty
            standard_price = bom_obj.product_id.standard_price * bom_obj.product_qty
        else:
            for bom_child in self.browse(cr, uid, child_ids, context=context):
                res = self._update_product_weight(cr, uid, bom_child, context=context)
                weight += res[0]
                weight_net += res[1]
                standard_price += res[2]
            cr.execute('update product_template set weight=%s, weight_net=%s, standard_price=%s where id=%s',(weight / bom_obj.product_qty,
                                                                                                              weight_net / bom_obj.product_qty, 
                                                                                                              standard_price / bom_obj.product_qty, 
                                                                                                              bom_obj.product_id.product_tmpl_id.id))

        return (weight, weight_net, standard_price)
    
    def write(self, cr, uid, ids, vals, context=None):
        result = super(bom, self).write(cr, uid, ids, vals, context=context)
        if result:
            for bom_obj in self.browse(cr, uid, ids, context=context):
                res = self._update_product_weight(cr, uid, bom_obj, context=context)
        return result
    
    def create(self, cr, uid, vals, context=None):
        result = super(bom, self).create(cr, uid, vals, context=context)
        bom_obj = self.browse(cr, uid, result, context=context)    
        res = self._update_product_weight(cr, uid, bom_obj, context=context)
        return result
    
    def _compute_type(self, cr, uid, ids, field_name, arg, context=None):
        """ 
        Sets particular method for the selected bom type.
        @param field_name: Name of the field
        @param arg: User defined argument
        @return:  Dictionary of values
        """
        res = dict(map(lambda x: (x, ''), ids))
        for line in self.browse(cr, uid, ids, context=context):
            if line.type in ('phantom', 'assembly') and not line.bom_id:
                res[line.id] = 'set'
                continue
            if line.bom_lines or line.type == 'phantom':
                continue
            if line.product_id.supply_method == 'produce':
                if line.product_id.procure_method == 'make_to_stock':
                    res[line.id] = 'stock'
                else:
                    res[line.id] = 'order'
        return res

    _inherit = 'mrp.bom'
    _columns = {
        'method': fields.function(_compute_type, string='Method', method=True, type='selection', 
                                  selection=[('', ''),('stock', 'On Stock'),('stock', 'On Stock'),('order', 'On Order'),('set', 'Set / Pack')]),
        'type': fields.selection([('normal', 'Normal BoM'), ('phantom', 'Sets / Phantom'), ('assembly', 'Assembly BoM')], 'BoM Type', required=True,
                                 help= "If a sub-product is used in several products, it can be useful to create its own BoM. "\
                                 "Though if you don't want separated production orders for this sub-product, select Set/Phantom as BoM type. "\
                                 "If a Phantom BoM is used for a root product, it will be sold and shipped as a set of components, \
                                 instead of being produced."),
          }
    
bom()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
