# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
###############################################################################

from osv import fields, osv

class product_product(osv.osv):

    _inherit = 'product.product'

    def _product_available(self, cr, uid, ids, field_names=None, arg=False, context=None):
        """
        For showing the bom stock value as 0 for the products with no bom
        @param self: The object pointer.
        @param ids: ids of the procurement order document
        @param context: standard dictionary
        @comment: If 'bom_stock' is there in the field_names list then if the product is having not bom the bom stock value was
                     shown as real stock value. This override modifies it to zero
        @return: returns 0 for the above conditions in the standard format dictionary for multi function fields
        """
        res = super(product_product, self)._product_available(cr, uid, ids, field_names, arg, context=context)
        bom_obj = self.pool.get('mrp.bom')
        if 'bom_stock' in field_names:
            for product_id, stock_qty in res.iteritems():
                product = self.browse(cr, uid, product_id, context=context)
                bom_ids = bom_obj._bom_find(cr, uid, product.id, product.uom_id.id, properties=[])
                if not bom_ids:
                    res[product_id]['bom_stock'] = 0
        return res

    def find_consumption_details(self, cr, uid, ids, context=None):
        """ 
        Finds the consumption details for the products with ids in 'ids'
         @param self: The object pointer.
         @param ids: ids of the procurement order document
         @param context: standard dictionary
         @return: returns a list of dictionaries that carries the information regarding the product consumption
        """
        if context is None:
            context = {}
        move_obj = self.pool.get('stock.move')
        product_ids = ids
        source_loc_id = context.get("source_loc_id", False)
        start_date = context.get("start_date", False)
        end_date = context.get("end_date", False)
        if source_loc_id:
            for product_id in product_ids:
                detail_list = []
                args = [('product_id', "=", product_id), ('location_id', '=', source_loc_id), ('state', '=', 'done')]
                start_date and args.append(('date', ">=", start_date + " 00:00:00"))
                end_date and args.append(('date', "<=", end_date + " 00:00:00"))
                move_ids = move_obj.search(cr, uid, args, context=context)
                for move in move_obj.browse(cr, uid, move_ids, context=context):
                    details = {}
                    #1. If move is having sale_line_id its assumed to be related with sale
                    if move.sale_line_id:
                        parent_product = move.product_id
                        details = {
                            'product_name': parent_product.name or "",
                            'reference': parent_product.default_code or "",
                            'type': 'sale',
                            "product_qty": move.product_qty,
                            'consumption': parent_product.name
                            }
                    #2. If move is having production_ids its assumed to be related with production using normal bom
                    if move.production_ids:
                        production = move.production_ids[0]
                        bom = production.bom_id and production.bom_id or False
                        if bom:
                            bom_type = bom.type
                            if bom_type == 'normal':
                                parent_product = production.product_id
                                product_name = parent_product.name
                                details = {
                                    'bom_type': bom_type,
                                    "product_name": product_name,
                                    "product_qty": move.product_qty,
                                    "parent_product": parent_product,
                                    'type': 'production',
                                    'consumption': product_name + '- ' + bom_type,
                                    'bom_type': bom_type,
                                    'reference': parent_product.default_code or ""
                                    }
                    #3. If move is having kit_id its assumed to be related with production using assembly bom
                    if move.kit_id:
                        production = move.kit_id
                        if production.state == 'done':
                            parent_product = production.product_id
                            bom = production.bom_id and production.bom_id or False
                            if bom:
                                bom_type = bom.type
                                if bom_type == 'assembly':
                                    product_name = parent_product.name
                                    product_qty = move.product_qty
                                    details = {
                                        'bom_type': bom_type,
                                        "product_name": product_name,
                                        "product_qty": move.product_qty,
                                        "parent_product": parent_product,
                                        'type': 'production',
                                        'consumption': product_name + '- ' + bom_type,
                                        'bom_type': bom_type,
                                        'reference': parent_product.default_code or ""
                                        }
                    details and detail_list.append(details)
                    details = {}
                consumpt_lines = []
                res = {}
                #4. This block analyses the details to aggregate them with reference to type of consumption
                for detail in detail_list:
                    product_name = detail.get('product_name')
                    consumption = detail.get('consumption')
                    if detail.get('type') == 'sale':
                        if consumption in res:
                            new_qty = res.get(consumption, {}).get('product_qty', 0) + detail.get('product_qty', 0)
                            res[consumption].update({'product_qty': new_qty, 'product_name': product_name})
                        else:
                            res[consumption] = {
                                'product_qty': detail.get('product_qty', 0),
                                'consumption': "%(product_name)s- sold separately"%detail,
                                'product_name': product_name,
                                'bom_type': detail.get('bom_type', ' '),
                                'type': 'Sold separately',
                                'reference': detail.get("reference", "")
                                }
                    if detail.get('type') == 'production':
                        if consumption in res:
                            new_qty = res.get(consumption, {}).get('product_qty', 0) + detail.get('product_qty', 0)
                            res[consumption].update({'product_qty': new_qty})
                        else:
                            res[consumption] = {
                                'product_qty': detail.get('product_qty', 0),
                                'consumption': consumption,
                                'product_name': product_name,
                                'bom_type': detail.get('bom_type', ' '),
                                'type': 'Production using: ' + detail.get('bom_type', ' ') + ' BOM',
                                'reference':detail.get("reference", "")
                                }
                consumpt_lines = res.values()
                total_qty = 0
                #5. Calculating percentage consumption
                for line in consumpt_lines:
                    total_qty += line.get('product_qty', 0)
                total_qty = float(total_qty)
                new_consumpt_lines = []
                for line in consumpt_lines:
                    line.update({'percentage': round(line.get('product_qty', 0) * 100 / total_qty, 2)})
                    new_consumpt_lines.append(line)
                consumpt_lines = new_consumpt_lines
                if not consumpt_lines:
                    consumpt_lines = False
                ret = {"source_loc_id": source_loc_id, 'detail_list': consumpt_lines}
                return ret
        else:
            return {}
        
    _columns = {
        'qty_available': fields.function(_product_available, method=True, type='float', string='Real Stock', 
                                         help="Current quantities of products in selected locations or all internal if none have been selected.", 
                                         multi='qty_available'),
        'virtual_available': fields.function(_product_available, method=True, type='float', string='Virtual Stock', 
                                             help="Future stock for this product according to the selected location or all internal if none have been \
                                             selected.Computed as: Real Stock - Outgoing + Incoming.", 
                                             multi='qty_available'),
        'immediately_usable_qty': fields.function(_product_available, method=True, type='float', string='Immediately Usable Stock', 
                                                  help="Quantities of products really available for sale. Computed as: Real Stock - Outgoing.",
                                                  multi='qty_available'),
        'bom_stock': fields.function(_product_available, method=True, type='float', string='BoM Stock Value', 
                                     help="Quantities of products,  useful to know how much of this product you could have. Computed as: \
                                     (Reference stock of This product + How much could I produce of this product)", 
                                     multi='qty_available'),
        }
    
    def _get_root_bom(self, cr, uid, bom_obj, context=None):
        
        if not bom_obj.bom_id:
		    return bom_obj
        else:
            return self._get_root_bom(cr, uid, bom_obj.bom_id, context=context)
                    
    def write(self, cr, uid, ids, vals, context=None):
       
        if isinstance(ids, (int, long)):
            ids = [ids]
        res = super(product_product, self).write(cr, uid, ids, vals, context=context)
        if res and (vals.get('weight', False) or vals.get('weight_net', False) or vals.get('standard_price', False)):
            updated_boms = []
            mrp_bom_pool = self.pool.get('mrp.bom')
            prod_ids = self.browse(cr, uid, ids, context=context)
            for prod in prod_ids:
                bom_ids = mrp_bom_pool.search(cr, uid, [('product_id', '=', prod.id)], context=context)
                for bom_obj in mrp_bom_pool.browse(cr, uid, bom_ids, context=context):
                    if bom_obj.bom_id:
                        parent_prod = bom_obj.bom_id.product_id
                        prod_ids.append(parent_prod)
                    root_bom = self._get_root_bom(cr, uid, bom_obj, context=context)
                    if root_bom and root_bom not in updated_boms:
                        updated_boms.append(root_bom)
                        res = mrp_bom_pool._update_product_weight(cr, uid, root_bom, context=context)
                        cr.execute('update product_template set weight=%s, weight_net=%s, standard_price=%s where id=%s',
                                   (res[0] / root_bom.product_qty, res[1] / root_bom.product_qty, res[2] / root_bom.product_qty, 
                                    root_bom.product_id.product_tmpl_id.id)
                                   )
        return res
    
product_product()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: