# -*- ecoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

import time

from tools.translate import _
from osv import fields, osv

class stock_move(osv.osv):
    _inherit = 'stock.move'

    def _action_explode(self, cr, uid, move, context=None):
        """ 
        Explodes pickings to produce many moves as per the bom lines for products assembly bom.
        @param move: Stock moves
        @constraints: explode takes place for pickings related to sale order
        @return: True
        """
        result = super(stock_move, self)._action_explode(cr, uid, move, context=context)
        bom_obj = self.pool.get('mrp.bom')
        move_obj = self.pool.get('stock.move')
        product_obj = self.pool.get('product.product')
        production_obj = self.pool.get('mrp.production')
        if move.product_id.supply_method == 'produce' and move.product_id.procure_method == 'make_to_order':
            bis = bom_obj.search(cr, uid, [('product_id', '=', move.product_id.id), ('bom_id', '=', False), ('type', '=', 'assembly')], context=context)
            if bis:
                factor = move.product_qty
                bom_point = bom_obj.browse(cr, uid, bis[0], context=context)
                res = bom_obj._bom_explode(cr, uid, bom_point, factor, [])
                dest = move and move.address_id and move.address_id.partner_id and move.address_id.partner_id.property_stock_customer and \
                    move.address_id.partner_id.property_stock_customer.id or False
                if move.sale_line_id:
                    #1. partial delivery is not allowed for kit type bom product
                    move.picking_id.write({'move_type': 'one'}, context=context)
                    #2. manually changing the state to confirmed for each exploded move to allow checking bom stock value
                        #related to mrp_jit
                    state = 'confirmed'
                    if move.state == 'assigned':
                        state = 'assigned'
                    if res[0] and res[0][0]:
                        line = res[0][0]
                        valdef = {
                            'picking_id': move.picking_id.id,
                            'product_id': line['product_id'],
                            'product_uom': line['product_uom'],
                            'product_qty': line['product_qty'],
                            'product_uos': line['product_uos'],
                            'product_uos_qty': line['product_uos_qty'],
                            'move_dest_id': move.id,
                            'state': state,
                            'name': line['name'],
                            'procurements': [],
                            'sale_line_id': False,
                            "kit_id": False,
                            "parent_bom_id": bis[0],
                            'sale_id': move.sale_line_id.order_id.id
                            }
                        if dest:
                            valdef['location_dest_id'] = dest
                        move.write(valdef, context=context)
                        
                    for line in res[0][1:]:
                        valdef = {
                            'picking_id': move.picking_id.id,
                            'product_id': line['product_id'],
                            'product_uom': line['product_uom'],
                            'product_qty': line['product_qty'],
                            'product_uos': line['product_uos'],
                            'product_uos_qty': line['product_uos_qty'],
                            'move_dest_id': move.id,
                            'state': state,
                            'name': line['name'],
                            'procurements': [],
                            'sale_line_id': False,
                            "kit_id": False,
                            "parent_bom_id": bis[0],
                            'sale_id': move.sale_line_id.order_id.id
                            }
                        if dest:
                            valdef['location_dest_id'] = dest
                        mid = move_obj.copy(cr, uid, move.id, default=valdef)
                        move_obj.action_assign(cr, uid, [mid])
                        prodobj = product_obj.browse(cr, uid, line['product_id'], context=context)

                    cr.commit()
        return result

    _columns={
        'sale_id': fields.many2one('sale.order', "Sale Reference"),
        'kit_id': fields.many2one('mrp.production', "Assembly"),
        'production_ids': fields.many2many('mrp.production', 'mrp_production_move_ids', 'move_id', 'production_id', 'Related Production'),
        'parent_bom_id': fields.many2one("mrp.bom", 'Related Bom')
        }
    
    def onchange_quantity(self, cr, uid, ids, product_id, product_qty, product_uom, product_uos, location_id=False):
        """ 
        On change of product quantity finds UoM and UoS quantities
        @param product_id: Product id
        @param product_qty: Changed Quantity of product
        @param product_uom: Unit of measure of product
        @param product_uos: Unit of sale of product
        @return: Dictionary of values
        """
        context = {}
        if location_id:
            context['location'] = [location_id]
        result = super(stock_move, self).onchange_quantity(cr, uid, ids, product_id, product_qty, product_uom, product_uos)
        if (not product_id) or (product_qty <= 0.0):
            return result
        product_obj = self.pool.get('product.product').browse(cr, uid, [product_id], context=context)[0]
        stock_location_obj = self.pool.get('stock.location')
        uom2 = product_obj.uom_id
        qty_virtual = product_obj.virtual_available
        qty_real = product_obj.qty_available
        if location_id:
            qty_virtual = stock_location_obj.read(cr, uid, location_id, ['stock_virtual'], {'product_id': product_id})['stock_virtual']
            qty_real = stock_location_obj.read(cr, uid, location_id, ['stock_real'], {'product_id': product_id})['stock_real']
        if (product_obj.type == 'product') and (qty_virtual * uom2.factor < product_qty * product_obj.uom_id.factor) \
                and (product_obj.procure_method == 'make_to_stock'):
            warning = {
                'title': _('Not enough stock !'),
                'message': _('You plan to move %.2f %s but you only have %.2f %s available !\nThe real stock is %.2f %s. (without reservations)') %
                     (product_qty, uom2 and uom2.name or product_obj.uom_id.name,
                      max(0, qty_virtual), product_obj.uom_id.name,
                      max(0, qty_real), product_obj.uom_id.name)
                }     
            result['warning'] = warning
                    
        if (product_obj.type == 'product') and product_obj.procure_method == 'make_to_order':
            bom_id = self.pool.get('mrp.bom').search(cr, uid, [('product_id', '=', product_obj.id), ('bom_id', '=', False)], context=context)
            if bom_id and (qty_virtual  < product_qty * product_obj.uom_id.factor):
                warning = {
                    'title': _('Not enough stock !'),
                    'message': _('You plan to move %.2f %s but you only have %.2f %s available !\nThe real stock is %.2f %s. (without reservations)') %
                         (product_qty, uom2 and uom2.name or product_obj.uom_id.name,
                          max(0, qty_virtual), product_obj.uom_id.name,
                          max(0, qty_real), product_obj.uom_id.name)
                    }     
                result['warning'] = warning                    
                    
        return result
    
    def onchange_product_id(self, cr, uid, ids, prod_id=False, loc_id=False, loc_dest_id=False, address_id=False, product_qty=0):
        """ 
        On change of product id, .
        @param prod_id: Changed Product id
        @param loc_id: Source location id
        @param loc_id: Destination location id
        @param address_id: Address id of partner
        @param product_qty: Product Quantity
        @return: Dictionary of values
        """
        warning = {}
        context = {}
        res = super(stock_move, self).onchange_product_id(cr, uid, ids, prod_id=prod_id, loc_id=loc_id, loc_dest_id=loc_dest_id, address_id=address_id)
        if not prod_id:
            return res
        if loc_id:
            context['location'] = [loc_id]
        product_obj = self.pool.get('product.product').browse(cr, uid, [prod_id], context=context)[0]
        stock_location_obj = self.pool.get('stock.location')
        uom2 = product_obj.uom_id
        qty_virtual = product_obj.virtual_available
        qty_real = product_obj.qty_available
        if loc_id:
            qty_virtual = stock_location_obj.read(cr, uid, loc_id, ['stock_virtual'], {'product_id': prod_id})['stock_virtual']
            qty_real = stock_location_obj.read(cr, uid, loc_id, ['stock_real'], {'product_id': prod_id})['stock_real']
        if (product_obj.type == 'product') and (qty_virtual * uom2.factor < product_qty * product_obj.uom_id.factor) \
                and (product_obj.procure_method == 'make_to_stock'):
            warning = {
                'title': _('Not enough stock !'),
                'message': _('You plan to move %.2f %s but you only have %.2f %s available !\nThe real stock is %.2f %s. (without reservations)') %
                     (product_qty, uom2 and uom2.name or product_obj.uom_id.name,
                      max(0, qty_virtual), product_obj.uom_id.name,
                      max(0, qty_real), product_obj.uom_id.name)
                }     
            res['warning'] = warning
            
        if (product_obj.type == 'product') and product_obj.procure_method == 'make_to_order':
            bom_id = self.pool.get('mrp.bom').search(cr, uid, [('product_id', '=', product_obj.id), ('bom_id', '=', False)], context=context)
            if bom_id and (qty_virtual  < product_qty * product_obj.uom_id.factor):
                warning = {
                    'title': _('Not enough stock !'),
                    'message': _('You plan to move %.2f %s but you only have %.2f %s available !\nThe real stock is %.2f %s. (without reservations)') %
                         (product_qty, uom2 and uom2.name or product_obj.uom_id.name,
                          max(0, qty_virtual), product_obj.uom_id.name,
                          max(0, qty_real), product_obj.uom_id.name)
                    }     
                res['warning'] = warning
        return res
    
    _defaults = {
        'date': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
        'date_expected': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
        }
    
stock_move()


class stock_picking(osv.osv):
    _inherit = "stock.picking"

    def do_partial(self, cr, uid, ids, partial_datas, context=None):
        """
        Creates and processes the assembly (MO) operation for delivery of product with assembly bom during sale
        @param self: The object pointer.
        @param ids: ids of the procurement order document
        @param context: standard dictionary
        @comment: Creates Production order for assembly of the product with assembly bom and
                    deliveres the product to customers making moves to ensure stock tracking
        @return: returns dictionary carrying delivered picking details
        """
        bom_obj = self.pool.get('mrp.bom')
        picking_obj = self.pool.get('stock.picking')
        move_obj = self.pool.get("stock.move")
        sale_obj = self.pool.get("sale.order")
        res = super(stock_picking, self).do_partial(cr, uid, ids, partial_datas, context=context)
        picking_details = res.values()
        picking_id = picking_details and picking_details[0].get('delivered_picking', False)
        if picking_id:
            #expecting the picking state will not be 'done'
            picking = picking_obj.browse(cr, uid, picking_id, context=context)
            sale_order = picking.sale_id
            if not sale_order:
                return res

            sale_obj.action_ship_end(cr, uid, [picking.sale_id.id], context=context)
            for line in sale_order.order_line:
                bom_ids = bom_obj.search(cr, uid, [('product_id', '=', line.product_id.id)], context=context)
                bom_id = bom_ids and bom_ids[0] or False
                if bom_id:
                    bom = bom_obj.browse(cr, uid, bom_id, context=context)
                    if bom and bom.type == 'normal':
                        move_ids = [move.id for move in picking.move_lines] or []
                        normal_move_ids = move_obj.search(cr, uid, [('id', 'in', move_ids), ('product_id', '=', line.product_id.id)], context=context)
                        ###Expecting 
                        #1. Sale order is having only one line with the product having normal bom
                        if normal_move_ids:
                            normal_move_id = normal_move_ids[0]#Refer:#1
                            normal_move = move_obj.browse(cr, uid, normal_move_id, context=context)
                            normal_move.write({'sale_id': line.order_id.id}, context=context)

        return res

    def action_confirm(self, cr, uid, ids, context=None):        
        res = super(stock_picking, self).action_confirm(cr, uid, ids, context=context)
        cr.commit()
        try:
            self.action_assign(cr, uid, ids)
        except Exception, e:
            pass    # Exception is not thrown needs to be checked.
        return True
    
    _defaults = {
        'date': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
        }

stock_picking()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: