# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

from osv import fields, osv
from tools.translate import _
import netsvc

class stock_partial_move_memory_out(osv.osv_memory):
    _inherit = "stock.move.memory.out"
    _columns = {
        'disassemble': fields.boolean('Disassemble'),
        'location_dest_id': fields.many2one('stock.location', 'Destination Locatoin'),
        }
        
stock_partial_move_memory_out()


class stock_partial_move_memory_in(osv.osv_memory):
    _inherit = "stock.move.memory.in"
    _columns = {
        'location_dest_id': fields.many2one('stock.location', 'Destination Locatoin'),
        }
        
stock_partial_move_memory_in()


class stock_partial_picking(osv.osv_memory):
    _inherit = "stock.partial.picking"
    
    def __create_partial_picking_memory(self, picking, pick_type):
        move_memory = super(stock_partial_picking, self). __create_partial_picking_memory(picking, pick_type)
        if pick_type == 'out':
            move_memory.update({'location_dest_id': picking.location_dest_id.id})                          
        return move_memory
    
    def add_to_stock(self, cr, uid, line, bom_id, context=None):
        production_obj = self.pool.get('mrp.production')
        disassemble_obj = self.pool.get('mrp.disassemble.location')
        mrp_data = {
            'disassemble': True,
            'product_qty': line.quantity,
            'product_id': line.product_id.id,
            'location_src_id': line.location_dest_id.id, 
            'location_dest_id': line.location_dest_id.id,
            }
        finish_location_ids = disassemble_obj.search(cr, uid, [('type', '=', 'incoming_shipment')], context=context)
        if finish_location_ids:
            mrp_data['location_dest_id'] = disassemble_obj.browse(cr, uid, finish_location_ids, context=context)[0].location_id.id
        mrp_data.update(production_obj.product_id_change(cr, uid, [], line.product_id.id, context=context)['value'])
        production_id = production_obj.create(cr, uid, mrp_data, context=context)
        wf_service = netsvc.LocalService('workflow')
        wf_service.trg_validate(uid, 'mrp.production', production_id, 'button_confirm', cr)
        self.pool.get('mrp.production').force_production(cr, uid, [production_id])
        wf_service.trg_validate(uid, 'mrp.production', production_id, 'button_produce', cr)
        production = production_obj.browse(cr, uid, production_id, context=context)
        for move in production.move_lines:
            self.pool.get('stock.move').action_consume(cr, uid, [move.id], move.product_qty, move.location_id.id, context=context)
            
        production_obj.action_produce(cr, uid, production_id, line.quantity, 'consume_produce', context=context)
        
        return True
    
    def get_bom(self, cr, uid, product_id, context=None):
        bom_obj = self.pool.get('mrp.bom')
        product = self.pool.get('product.product').browse(cr, uid, product_id, context=context)
        bom_id = bom_obj._bom_find(cr, uid, product.id, product.uom_id and product.uom_id.id, [])
        return bom_id
    
    def do_partial(self, cr, uid, ids, context=None):
        
        data = self.browse(cr, uid, ids[0], context=context)
        if context is None:
            context = {}
        for line in data.product_moves_out:
            if line.disassemble:
                bom_id = self.get_bom(cr, uid, line.product_id.id)
                if bom_id:
                    self.add_to_stock(cr, uid, line, bom_id, context=context)
                else:
                    raise osv.except_osv(_("No BoM found"), _("Please configure a BoM for product %s ")%line.product_id.name)
        pick_obj = self.pool.get('stock.picking')
        move_obj = self.pool.get('stock.move')
        location_obj = self.pool.get('stock.location')
        picking_ids = context.get('active_ids', False)
        partial = self.browse(cr, uid, ids[0], context=context)
        partial_datas = {'delivery_date': partial.date}
        for pick in pick_obj.browse(cr, uid, picking_ids, context=context):
            picking_type = self.get_picking_type(cr, uid, pick, context=context)
            moves_list = picking_type == 'in' and partial.product_moves_in or partial.product_moves_out

            for move in moves_list:
                partial_datas['move%s' % (move.move_id.id)] = {
                    'product_id': move.id, 
                    'product_qty': move.quantity, 
                    'product_uom': move.product_uom.id, 
                    'prodlot_id': move.prodlot_id.id, 
                    }
                if (picking_type == 'in') and (move.product_id.cost_method == 'average'):
                    partial_datas['move%s' % (move.move_id.id)].update({
                        'product_price': move.cost, 
                        'product_currency': move.currency.id, 
                        })
        pick_obj.do_partial(cr, uid, picking_ids, partial_datas, context=context)
        return {}

stock_partial_picking()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: