# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
{
    "name" : "Tax Exemption Management",
    "version" : "1.0",
    "author" : 'Novapoint Group LLC',
    "description": """
This module helps manage and track sales tax exemptions at the partner level. The module has the following
capabilities:
• Individual states/or other tax jurisdictions certificate of exemptions are tracked
• Exemption reasons are tracked for each entry
• The exemption identification number is tracked for each entry
• Effective dates are tracked if available for each entry since exemptions must be renewed from time to
time and expire
• The “State” (aka status) of an exemption is tracked.
• Allows historical exemption numbers (multi-entries per state) to be tracked over time to support audit
activities.
• Each entry can be pointed to the image of the Certificate either held in a third party document system
or in OpenERP if document management is installed (Auditors will want to see the signed certificate
provided by the customer to the company during an audit)
• The name of the exemption form used to validate the exemption
• Track the name of the signer of the certificate for a buying company

This module is modeled after the fields from the “Streamlined Sales and Use Tax Agreement” multi-state
Certificate of Exemption form.

This adds additional fields to the partner record.

General Rules (see your tax advisor and accountant)
As a general rule, sellers are required to maintain proper records for exempt transactions and provide those
records to member states and auditors when requested. These certificates may be provided in paper or
electronic form. A seller is typically relieved of the responsibility for collecting and remitting sales tax on the
sale or sales that a purchaser provided a seller an exemption certificate. These are based on certain condition
which the seller should investigate. Each State/Tax jurisdiction has different rules and a customer may be
exempt from sales taxes (for resale goods) in one tax jurisdiction and non-exempt in the other. Over time
this module builds exemption histories and helps when a company is audited for their historical sales tax
compliance. Accounting Auditors look at accurate tracking of exemptions to validate if taxes have been charged
appropriately.

Any company in the US that sells goods and services B2B should track exemptions. And companies themselves
should track their own exemption status. This module is designed to help monitor and track exemption
certificates for both customers and for a “Sellers” own accounts.

*** Future development plans *** (Feel free to enhance this module – or donate funds to
www.novapointgroup.com to enhance it for you).
1. Improve the module to incorporate workflow and notifications to help improve management of
certificate. Phase I – is setup to allow tracking of exemption certificates. Phase II – will help to facilitate
requesting exemption certificates. Including tracking the interactions/correspondence.
2. Create and track applicable exemption status options at the individual state/tax jurisdiction level. Each
state may not support the full list of “exemption reasons”.""",
    "category" : "Generic Modules/Accounting",
    "website" : "http://www.novapointgroup.com/",
    "depends" : ["base"],
    "init_xml" : [],
    "demo_xml" : [
        "partner_exemption_us_demo.xml"
    ],
    "update_xml" : [
        "partner_exemption_us_view.xml",
        "partner_view.xml",
#        "security/ir.model.access.csv",
    ],
    "test" : [],
    "active": False,
    "installable": True,
}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
