# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import time

from osv import osv, fields

class res_partner_exemption_rejection(osv.osv):
    _name = "res.partner.exemption.rejection"
    _description = "Exemption Rejection Code"
    _columns = {
        'name': fields.char('Reason', size=32, required=True),
        'code': fields.char('code', size=32)
    }

    def name_get(self, cr, uid, ids, context=None):
        if not ids:
            return []
        reads = self.read(cr, uid, ids, ['name', 'code'], context=context)
        res = []
        for record in reads:
            name = record['name']
            if record['code']:
                name = '(' + record['code'] + ')' + ' ' + name
            res.append((record['id'], name))
        return res

res_partner_exemption_rejection()

class res_partner_exemption_reason(osv.osv):
    _name = 'res.partner.exemption.reason'
    _description = 'Exemption Reason Code'
    _columns = {
        'name': fields.char('Description', size=64, required=True),
        'code': fields.char('Code', size=1)
    }

    def name_get(self, cr, uid, ids, context=None):
        if not ids:
            return []
        reads = self.read(cr, uid, ids, ['name', 'code'], context=context)
        res = []
        for record in reads:
            name = record['name']
            if record['code']:
                name = '(' + record['code'] + ')' + ' ' + name
            res.append((record['id'], name))
        return res

res_partner_exemption_reason()

class tax_jurisdiction(osv.osv):
    _name = 'tax.jurisdiction'
    _description = 'Tax Jurisdiction'
    _columns = {
        'name': fields.char('Name', size=32, required=True),
        'country_id': fields.many2one('res.country', 'Country', help="The country that issued exemption"),
        'state_id': fields.many2one('res.country.state', 'State', help="The state that issued exemption"),
        'city': fields.char('City', size=64),
    }

tax_jurisdiction()

class res_partner_exemption(osv.osv):
    _name = 'res.partner.exemption'
    _description = 'Partner Exemption'
    _rec_name = 'number'
    _columns = {
        'description': fields.char('Decription', size=32, help="Describes the certificate"),
        'partner_id': fields.many2one('res.partner', 'Partner'),
        'number': fields.char('Exemption Number', size=32, help="Unique Identifier of an exemption certificate", required=True),
        'number_type': fields.selection([('taxpayer', 'Exemption Number/Taxpayer ID'), ('fein', 'FEIN'), ('license', 'Driver License Number'), ('diplomat', 'Foreign Diplomat Number')], 'Exemption Number Type', required=True, help="Defines the type of the exemption number"),
        'type': fields.selection([('blanket', 'Blanket'), ('single', 'Single')], 'Certificate Type', help="'Blanket' Certificates cover all the purchases and 'Single' Certificates are used for single invoice or purchase order"),
        'invoice_purchase_no': fields.char('Invoice/Purchase No', size=32, help="Required if certificate type is /'single/' to be only used for single invoice/purchase order"),
        'code_id': fields.many2one('res.partner.exemption.reason', 'Exemption Reason Code', help="Indicates the reason for exemption"),
        'state': fields.selection([('draft', 'Draft'), ('active', 'Active'), ('pending', 'Pending Review'), ('expired', 'Expired'), ('rejected', 'Rejected')], 'Status', required=True, help="Defines the status of the certificate:'Active' certificates can be used to exempt partner from tax, 'Pending' certificates are without the copy of certificate received from the partner, 'Expired' certificate is the one in which ending date has passed in any of the issuing state, 'Rejected' certificate is the one which is rejected"),
        'effective_date': fields.date('Effective Date', help="Issue date of the exemption certificate"),
        'description_reason': fields.text('Description of Reason ', size=64, help="General description of exemption reason"),
        'description_property': fields.text('Description of Tangible Property ', size=64, help="General description of tangible property or taxable services to be purchased by the partner"),
        'country_id': fields.many2one('res.country', 'Issuing Country', help='The country which issued exemption certificate.'),
        'source': fields.char('Source', size=32, help="Source of the certificate"),
        'user_id': fields.many2one('res.users', 'Reviewed By', help='The user who reviewed the exemption certificate document'),
        'rejection_id': fields.many2one('res.partner.exemption.rejection', 'Rejection Reason', help='The reason why a submitted certificate was rejected.'),
        'expiration_date': fields.date('Expiration Date', help="Expiration date of the exemption certificate entry"),
        'received_date': fields.date('Received Date', help="Date when the copy of exemption certificate document was received from the buyer."),
        'not_expire':fields.boolean('Does Not Expire', help="Indicates if the certificate doesn't expires."),
        'received':fields.boolean('Received', help="Indicates if a certificate document was received and is on file."),
        'document_id': fields.many2one('ir.attachment', 'Document', help="The actual location of the the certificate document"),
        'jurisdiction_id': fields.many2one('tax.jurisdiction', 'Jurisdiction', required=True, help='The Tax Jurisdiction for which the exemption is valid.'),
        'review_state': fields.selection([('draft', 'Draft'), ('pending', 'Pending Review'), ('accepted', 'Accepted'), ('disputed', 'Disputed'), ('rejected', 'Rejected')], 'Review Status', required=True, help="The status of the review of the document"),
        'active':fields.boolean('Active', help="If the active field is set to False, it will allow you to hide the exemption without removing it.")
    }

    _defaults = {
        'type': 'blanket',
        'active': True,
        'state': 'draft',
        'review_state': 'draft'
    }
res_partner_exemption()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
