# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

from osv import fields, osv

class stock_packages(osv.osv):

    def process_package(self, cr, uid, ids, context=None):
        return True
    
    def _get_decl_val(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        if context is None:
            context = {}
        sum = 0
        for rec in self.browse(cr, uid, ids, context=context):
            for item in rec.package_item_ids:
                sum += item.cost or 0.0
            res[rec.id] = sum
        return res
    
    def _get_highvalue(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        if context is None:
            context = {}
        highvalue = False
        for rec in self.browse(cr, uid, ids, context=context):
            if rec.decl_val > 1000:
                highvalue = True
            res[rec.id] = highvalue
        return res

    _inherit = "stock.packages"
    _columns = {
        'shipment_digest': fields.text('ShipmentDigest'),
        'control_log_receipt': fields.binary('Control Log Receipt'),
        'package_type': fields.many2one('shipping.package.type','Package Type'),
        'highvalue': fields.function(_get_highvalue, method=True, type='boolean', string='High Value',
                                     store={
                            'stock.packages': (lambda self, cr, uid, ids, c={}: ids, ['package_item_ids'], 20),
                    }),
        'decl_val': fields.function(_get_decl_val, method=True, string='Declared Value', type='float', help='The declared value of the package.',
                                    store={
                            'stock.packages': (lambda self, cr, uid, ids, c={}: ids, ['package_item_ids'], 10),
                    })
        }
    
    def print_control_receipt_log(self, cr, uid, ids, context=None):
        if not ids: return []
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'control.log.receipt.print',
            'datas': {
                'model': 'stock.packages',
                'id': ids and ids[0] or False,
                'ids': ids and ids or [],
                'report_type': 'pdf'
                },
            'nodestroy': True
            }
        
stock_packages()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: