# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

import time

from report import report_sxw
import pooler

class summary_report_print(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context):
        super(summary_report_print, self).__init__(cr, uid, name, context)
        db_pool = pooler.get_pool(self.cr.dbname)
        self.localcontext.update({
            'time': time,
            'get_items': self.get_items,
            'get_picker': self.get_picker,
            'get_origin': self.get_origin,
        })
        
    def get_picker(self, id):
        picker = pooler.get_pool(self.cr.dbname).get('res.users').read(self.cr, self.uid, id, ['name'])['name']
        return str(picker)
    
    def get_origin(self, id):
        ret = []
        picking_pool = pooler.get_pool(self.cr.dbname).get('stock.picking')
        ship_ids = picking_pool.search(self.cr, self.uid, [('assigned_picker', '=', id), ('state', 'in', ['assigned'])])
        for pick in picking_pool.browse(self.cr, self.uid, ship_ids):
            ret.append(pick.origin or '')
            
        ret = [x for x in ret if x not in [False, '']]
        return ', '.join(ret)
    
    def get_items(self, id):
        ret = {}
        orig = {}
        picking_pool = pooler.get_pool(self.cr.dbname).get('stock.picking')
        ship_ids = picking_pool.search(self.cr, self.uid, [('assigned_picker', '=', id), ('state', 'in', ['assigned'])])
        for pick in picking_pool.browse(self.cr, self.uid, ship_ids):
            for move in pick.move_lines:
                if (move.product_id.name, move.location_id.name, move.product_id.default_code) not in orig:
                    orig[(move.product_id.name, move.location_id.name, move.product_id.default_code)] = []
                ret[(move.product_id.name, move.location_id.name, move.product_id.default_code)] = ret.get((move.product_id.name, move.location_id.name, move.product_id.default_code), 0) + move.product_qty
                orig[(move.product_id.name, move.location_id.name, move.product_id.default_code)].append(pick.origin)
        for key in ret:
            ret[key] = (ret[key], orig[key])
        temp_res = [(x, y) for x, y in ret.items()]
        temp_res = sorted(temp_res, key=lambda x: x[0][1])
        return temp_res

report_sxw.report_sxw('report.pick_list.report', 'stock.picking', 
                      'addons/stock_assigned_picker/report/pick_list_report.rml', 
                      parser=summary_report_print)
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: