# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

from datetime import datetime

from osv import osv
from tools.translate import _
import netsvc

class mrp_production(osv.osv):
    _inherit = 'mrp.production'
    
    def action_confirm(self, cr, uid, ids, context=None):
        """
        Confirms production order.
        Here rewrite this code to add functionality to split the products to consume line according to stock availability
        @return: Newly generated picking Id.
        Need to rewrite this code by using super
        """
        picking_id = False
        proc_ids = []
        seq_obj = self.pool.get('ir.sequence')
        pick_obj = self.pool.get('stock.picking')
        move_obj = self.pool.get('stock.move')
        proc_obj = self.pool.get('procurement.order')
        wf_service = netsvc.LocalService("workflow")
        for production in self.browse(cr, uid, ids):
            if not production.product_lines:
                self.action_compute(cr, uid, [production.id])
                production = self.browse(cr, uid, [production.id])[0]
            routing_loc = None
            pick_type = 'internal'
            address_id = False
            if production.bom_id.routing_id and production.bom_id.routing_id.location_id:
                routing_loc = production.bom_id.routing_id.location_id
                if routing_loc.usage <> 'internal':
                    pick_type = 'out'
                address_id = routing_loc.address_id and routing_loc.address_id.id or False
                routing_loc = routing_loc.id
            pick_name = seq_obj.get(cr, uid, 'stock.picking.' + pick_type)
            picking_id = pick_obj.create(cr, uid, {
                'name': pick_name,
                'origin': (production.origin or '').split(':')[0] + ':' + production.name,
                'type': pick_type,
                'move_type': 'one',
                'state': 'auto',
                'address_id': address_id,
                'auto_picking': self._get_auto_picking(cr, uid, production),
                'company_id': production.company_id.id,
                'explode_multi_locations': False
                })
            source = production.product_id.product_tmpl_id.property_stock_production.id
            data = {
                'name': 'PROD:' + production.name,
                'date': production.date_planned,
                'product_id': production.product_id.id,
                'product_qty': production.product_qty,
                'product_uom': production.product_uom.id,
                'product_uos_qty': production.product_uos and production.product_uos_qty or False,
                'product_uos': production.product_uos and production.product_uos.id or False,
                'location_id': source,
                'location_dest_id': production.location_dest_id.id,
                'move_dest_id': production.move_prod_id.id,
                'state': 'waiting',
                'company_id': production.company_id.id,
            }
            res_final_id = move_obj.create(cr, uid, data)

            self.write(cr, uid, [production.id], {'move_created_ids': [(6, 0, [res_final_id])]})
            moves = []
            for line in production.product_lines:
                move_id = False
                newdate = production.date_planned
                total_product = remaining_product = line.product_qty
                if line.product_id.type in ('product', 'consu'):
                    if line.product_id.type == 'consu':
                        res_dest_id = move_obj.create(cr, uid, {
                            'name': 'PROD:' + production.name,
                            'date': production.date_planned,
                            'product_id': line.product_id.id,
                            'product_qty': remaining_product, #min(remaining_product, multi_location.qty_available),
                            'product_uom': line.product_uom.id,
                            'product_uos_qty': line.product_uos and line.product_uos_qty or False,
                            'product_uos': line.product_uos and line.product_uos.id or False,
                            'location_id': production.location_src_id.id,
                            'location_dest_id': source,
                            'move_dest_id': res_final_id,
                            'state': 'waiting',
                            'company_id': production.company_id.id,
                            })
                        moves.append(res_dest_id)
                        remaining_product = 0
                    else:
                        multi_location_ids = []
                        for multi_location in line.product_id.multi_location_ids:
                            if multi_location.storage_type == 'primary':
                                multi_location_ids.append(multi_location.id)
                        for multi_location in line.product_id.multi_location_ids:
                            if multi_location.storage_type != 'primary':
                                multi_location_ids.append(multi_location.id)
                        for multi_location in self.pool.get('stock.multi.location').browse(cr, uid, multi_location_ids):
                            if remaining_product > 0 and multi_location.qty_available > 0:
                                res_dest_id = move_obj.create(cr, uid, {
                                    'name': 'PROD:' + production.name,
                                    'date': production.date_planned,
                                    'product_id': line.product_id.id,
                                    'product_qty': min(remaining_product, multi_location.qty_available),
                                    'product_uom': line.product_uom.id,
                                    'product_uos_qty': line.product_uos and line.product_uos_qty or False,
                                    'product_uos': line.product_uos and line.product_uos.id or False,
                                    'location_id': multi_location.location_name.id or production.location_src_id.id,
                                    'location_dest_id': source,
                                    'move_dest_id': res_final_id,
                                    'state': 'waiting',
                                    'company_id': production.company_id.id,
                                    })
                                moves.append(res_dest_id)
                                remaining_product = remaining_product - min(remaining_product, multi_location.qty_available)
                    if remaining_product > 0:
                        model_obj = self.pool.get("ir.model.data")
                        (model, res_id) = model_obj.get_object_reference(cr, uid, 'stock_multilocation_picking', 'stock_location_shortage') 
                        loc_id = (model == "stock.location") and  res_id
                        if not loc_id:
                            raise osv.except_osv('Error', 'Primary/Shortage location for the product [%s] %s is not defined.'
                                                 %(line.product_id.default_code, line.product_id.name) )
                        res_dest_id = move_obj.create(cr, uid, {
                            'name': 'PROD:' + production.name,
                            'date': production.date_planned,
                            'product_id': line.product_id.id,
                            'product_qty': remaining_product,
                            'product_uom': line.product_uom.id,
                            'product_uos_qty': line.product_uos and line.product_uos_qty or False,
                            'product_uos': line.product_uos and line.product_uos.id or False,
                            'location_id': loc_id or routing_loc or production.location_src_id.id,
                            'location_dest_id': source,
                            'move_dest_id': res_final_id,
                            'state': 'waiting',
                            'company_id': production.company_id.id,
                            })
                        moves.append(res_dest_id)
                    move_id = move_obj.create(cr, uid, {
                        'name': 'PROD:' + production.name,
                        'picking_id':picking_id,
                        'product_id': line.product_id.id,
                        'product_qty': line.product_qty,
                        'product_uom': line.product_uom.id,
                        'product_uos_qty': line.product_uos and line.product_uos_qty or False,
                        'product_uos': line.product_uos and line.product_uos.id or False,
                        'date': newdate,
                        'move_dest_id': res_dest_id,
                        'location_id': production.location_src_id.id,
                        'location_dest_id': routing_loc or production.location_src_id.id,
                        'state': 'waiting',
                        'company_id': production.company_id.id,
                        })
                proc_id = proc_obj.create(cr, uid, {
                    'name': (production.origin or '').split(':')[0] + ':' + production.name,
                    'origin': (production.origin or '').split(':')[0] + ':' + production.name,
                    'date_planned': newdate,
                    'product_id': line.product_id.id,
                    'product_qty': line.product_qty,
                    'product_uom': line.product_uom.id,
                    'product_uos_qty': line.product_uos and line.product_qty or False,
                    'product_uos': line.product_uos and line.product_uos.id or False,
                    'location_id': production.location_src_id.id,
                    'procure_method': line.product_id.procure_method,
                    'move_id': move_id,
                    'company_id': production.company_id.id,
                    })
                self.write(cr, uid, [production.id], {'picking_id': picking_id})
                wf_service.trg_validate(uid, 'procurement.order', proc_id, 'button_confirm', cr)
                proc_ids.append(proc_id)
            wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
            self.write(cr, uid, [production.id], {'picking_id': picking_id, 'move_lines': [(6, 0, moves)], 'state': 'confirmed'})
            self.pool.get('stock.move').action_assign(cr, uid, moves, None)
            message = _("Manufacturing order '%s' is scheduled for the %s.") % (
                production.name,
                datetime.strptime(production.date_planned, '%Y-%m-%d %H:%M:%S').strftime('%m/%d/%Y'),
            )
            self.log(cr, uid, production.id, message)
        return picking_id

    def change_qty(self, cr, uid, id, qty, context=None):
        wf_service = netsvc.LocalService("workflow")
        prod = self.browse(cr, uid, id, context=context)
        self.pool.get('stock.picking').action_cancel(cr, uid, [prod.picking_id.id], context=context )
        for move in prod.move_lines:
            self.pool.get('stock.move').action_cancel(cr, uid, [move.id], context=context)
        self.write(cr, uid, [prod.id], {'move_lines': [(6, 0, [])], 'move_lines2': [(6, 0, [])], 'picking_id': False}, context=context)
        for created_move in prod.move_created_ids:
            self.pool.get('stock.move').action_cancel(cr, uid, [created_move.id], context=context)
        self.write(cr, uid, [prod.id], {'state': 'draft', 'product_qty': qty}, context=context)
        self.action_compute(cr, uid, [prod.id])
        self.action_confirm(cr, uid, [prod.id])        

    def force_production(self, cr, uid, ids, *args):
        res = super(mrp_production, self).force_production(cr, uid, ids, args)
        self.action_ready(cr, uid, ids)
        return True
    
mrp_production()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: