# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

from osv import fields, osv
import netsvc

class stock_move(osv.osv):
    _inherit = "stock.move"

    def _get_order_no(self, cr, uid, product):
        eval = -1
        for multi_loc in product.multi_location_ids:
            if multi_loc.evaluation_order > eval:
                eval = multi_loc.evaluation_order
        return eval + 1

    def _get_storage_type(self, cr, uid, product):
            for multi_loc in product.multi_location_ids:
                if multi_loc.storage_type == 'primary':
                    return 'secondary'
            return 'primary'

    def _create_multilocation(self, cr, uid, ids, vals, context=None):
        if vals.get('move_dest_id', False) or  (vals.get('state', False) and vals['state'] <> "cancel" ):
            warehouse_obj = self.pool.get('stock.warehouse')
            warehouse_ids = warehouse_obj.search(cr, uid, [], context=context)
            lot_stock_ids = []
            for lot_stock_id in warehouse_obj.read(cr, uid, warehouse_ids, ['lot_stock_id']):
                lot_stock_ids.append(lot_stock_id['lot_stock_id'][0])
            if isinstance(ids, (int, long)):
                ids = [ids]
            for move in self.browse(cr, uid, ids, context=context):
                if move.location_dest_id.usage != 'internal' or move.location_dest_id.id in lot_stock_ids:
                    continue
                multi_loc_obj = self.pool.get('stock.multi.location')
                search_list = [('product_id', '=', move.product_id.id), ('location_name', '=', move.location_dest_id.id)]
                if not multi_loc_obj.search(cr, uid, search_list, context=context):
                    evaluation_order = 1
                    storage_type = "overstock"
                    for multi_location in move.product_id.multi_location_ids:
                        if multi_location.evaluation_order >= evaluation_order:
                            evaluation_order = multi_location.evaluation_order + 1
                    multi_loc_obj.create(cr, uid, {
                        'valuation_order': evaluation_order,
                        'storage_type': storage_type,
                        'warehouse': multi_loc_obj._get_shop_id(cr, uid, [], context=context),
                        'location_name': move.location_dest_id.id,
                        'product_uom': move.product_uom.id,
                        'min_level': 0,
                        'target_level': 0,
                        'qty_multiple': 1,
                        'location_active': True,
                        'product_id':move.product_id.id,
                        }, context=context)
        return True

    def write(self, cr, uid, ids, vals, context=None):
        if isinstance(ids, (int, long)):
            ids = [ids]
        res=super(stock_move, self).write(cr, uid, ids, vals, context=context)
        if res:
            self._create_multilocation(cr, uid, ids, vals, context)
        return res

    def create(self, cr, uid, vals, context=None):
        res=super(stock_move, self).create(cr, uid, vals, context)
        if res:
            self._create_multilocation(cr, uid, [res], vals, context)
        return res

    def _stock_available(self, cr, uid, ids, name, arg, context=None):
        res = {}
        for obj in self.browse(cr, uid, ids, context=context):
            qty = obj.product_qty
            loc_obj = self.pool.get('stock.location')
            stock_vals = loc_obj.read(cr, uid, obj.location_id.id, ['stock_real', 'stock_virtual'], context={'product_id': obj.product_id.id})
            if obj.product_id.type != 'consu' and stock_vals['stock_real'] < qty:
                res[obj.id] = False
            else:
                res[obj.id] = True
        return res

    _columns = {
        'stock_available': fields.function(_stock_available, method=True, type='boolean', string='Stock Available')
        }
    
    def check_assign(self, cr, uid, ids, context=None):
        ret = []
        move_data = self.read(cr, uid, ids, ['picking_id'], context=context)
        for move in move_data:
            picking_id = move['picking_id'] and move['picking_id'][0]
            production_ids = self.pool.get('mrp.production').search(cr, uid, [('picking_id', '=', picking_id)], context=context)
            if picking_id and not production_ids:
                ret.append(move['id'])
                continue
            if not picking_id:
                ret.append(move['id'])
                continue
        return super(stock_move, self).check_assign(cr, uid, ret, context=context)
    
    def action_consume(self, cr, uid, ids, product_qty, location_id=False, context=None): 
        """ 
        Consumed product with specific quatity from specific source location.
        @param product_qty: Consumed product quantity
        @param location_id: Source location
        @return: Consumed lines
        """       
        res = []
        production_obj = self.pool.get('mrp.production')
        wf_service = netsvc.LocalService("workflow")
        for move in self.browse(cr, uid, ids, context=context):
            new_moves = super(stock_move, self).action_consume(cr, uid, [move.id], product_qty, move.location_id.id, context=context)
            production_ids = production_obj.search(cr, uid, [('move_lines', 'in', [move.id])])
            for prod in production_obj.browse(cr, uid, production_ids, context=context):
                if prod.state == 'confirmed':
                    production_obj.force_production(cr, uid, [prod.id])
                wf_service.trg_validate(uid, 'mrp.production', prod.id, 'button_produce', cr)
            for new_move in new_moves:
                if new_move == move.id:
                    continue #This move is already there in move lines of production order
                production_obj.write(cr, uid, production_ids, {'move_lines': [(4, new_move)]}, context=context)
                res.append(new_move)
        return res
    
stock_move()

class stock_picking(osv.osv):
    _inherit = "stock.picking"
    _columns = {
        'explode_multi_locations': fields.boolean('Explode Multi Location')
        }
    
    _defaults = {
        'explode_multi_locations': True
        }

    def explode_multilocation(self, cr, uid, ids):
        for pick in self.browse(cr, uid, ids):
            if not (pick.explode_multi_locations and pick.type == "out"):
                continue
            for move in pick.move_lines:
                total_product = remaining_product = move.product_qty
                if total_product < 0:
                    continue
                for multi_location in move.product_id.multi_location_ids:
                    multi_location_id = multi_location.location_name.location_id and multi_location.location_name.location_id.id or False 
                    if remaining_product >0 and multi_location_id == move.location_id.id and multi_location.qty_available > 0:
                        default_vals = {
                            'location_id': multi_location.location_name.id,
                            'product_qty': min(remaining_product, multi_location.qty_available),
                            'location_dest_id': move.location_dest_id.id,
                            'date_expected': move.date_expected,
                            'state': move.state
                            }
                        remaining_product = remaining_product - min(remaining_product, multi_location.qty_available)
                        last_move_created_id = self.pool.get("stock.move").copy(cr, uid, move.id, default_vals)
                if remaining_product > 0:
                    cr.execute("update stock_move set product_qty = %s where id = %s" %(str(remaining_product), str(move.id)))
                else:
                    cr.execute("update procurement_order set move_id = %s where move_id = %s" %(last_move_created_id, move.id))  
                    cr.execute("delete from stock_move where id = %s" %str(move.id))
        return True

    def action_assign(self, cr, uid, ids, *args):
        self.explode_multilocation(cr, uid, ids)
        ret = super(stock_picking, self).action_assign(cr, uid, ids, *args)
        return ret

    def draft_force_assign(self, cr, uid, ids, *args):
        self.explode_multilocation(cr, uid, ids)
        ret = super(stock_picking, self).draft_force_assign(cr, uid, ids, *args)
        return ret

    def draft_validate(self, cr, uid, ids, *args):
        self.explode_multilocation(cr, uid, ids)
        ret = super(stock_picking, self).draft_validate(cr, uid, ids, *args)
        return ret

stock_picking()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: