# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

from osv import fields, osv

class wizard_replenish(osv.osv_memory):

    def _get_product(self, cr, uid, context=None):
        if context is None:
            context = {}
        if 'active_id' in context:
            replenish = self.pool.get('multilocation.replenish').browse(cr, uid, context['active_id'], context=context)
        return replenish.product_name.id

    def ok_action(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        vals = {}
        data = self.pool.get('wizard.replenish').browse(cr, uid, ids[0])
        if 'active_id' in context:
            replenish = self.pool.get('multilocation.replenish').browse(cr, uid, context['active_id'])
            if data.qty > 0 and data.qty <= replenish.amt_to_move:
                vals.update({
                    'product_id': replenish.product_name.id,
                    'location_id': replenish.replenish_loc.id,
                    'location_dest_id': replenish.primary_loc.id,
                    'product_qty': data.qty,
                    'product_uom': replenish.product_name.uom_id.id,
                    'state': 'done'
                    })
                picking_id = self.pool.get('stock.picking').create(cr, uid, {
                    'origin': replenish.replenish_move,
                    'type': 'internal',
                    'state': 'done',
                    'move_type': 'direct',
                    }, context=context)
                vals['picking_id'] = picking_id
                vals['name'] = replenish.replenish_move
                move_id = self.pool.get('stock.move').create(cr, uid, vals, context=context)
                replenish.write({'amt_moved': data.qty, 'state': 'done'}, context=context)
            if data.remove_sec_loc:
                if replenish.amt_left <= 0:
                    multi_loc_obj = self.pool.get('stock.multi.location')
                    stock_loc_id = multi_loc_obj.search(cr, uid, [('location_name', '=', replenish.replenish_loc.id)], context=context)
                    multi_loc_obj.unlink(cr, uid, stock_loc_id, context=context)
        return {}

    def _get_left_qty(self, cr, uid, context=None):
        if context is None:
            context = {}
        if 'active_id' in context:
            replenish = self.pool.get('multilocation.replenish').browse(cr, uid, context['active_id'], context=context)
            if replenish.amt_left <=0:
                return True
            else:
               return False

    def _get_qty(self, cr, uid, context=None):
        if context is None:
            context = {}
        if 'active_id' in context:
            replenish = self.pool.get('multilocation.replenish').browse(cr, uid, context['active_id'], context=context)
            return replenish.amt_to_move

    _name = "wizard.replenish"
    _columns = {
        'product_id': fields.many2one('product.product', 'Product'),
        'remove_sec_loc': fields.boolean('Remove The Secondory Location'),
        'qty': fields.integer('Amount to Replenish'),
        'real_lt_qty': fields.boolean('Real stock lt QTY'),
        }
    _defaults = {
        'product_id': _get_product,
        'qty': _get_qty,
        'real_lt_qty': _get_left_qty,
        'remove_sec_loc': True
        }

wizard_replenish()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: