# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

import time

from osv import osv
from tools.translate import _

class replenish_update(osv.osv_memory):
    _name = "replenish.update"

    def make_replenish(self, cr, uid, ids, context=None):
        id_list = []
        if context is None:
            context = {}
        replenish_obj = self.pool.get('multilocation.replenish')
        if context.get('active_model', False) == 'product.product':
            prod_ids = context['active_ids']
            repl_ids = replenish_obj.search(cr, uid, [('product_name', 'in', prod_ids), ('state', '!=', 'done')], context=context)
            replenish_obj.unlink(cr, uid, repl_ids)
        else:
            prod_ids = self.pool.get('product.product').search(cr, uid, [], context=context)
            repl_ids = replenish_obj.search(cr, uid, [])
            replenish_obj.unlink(cr, uid, repl_ids)
        for prod_id in prod_ids:                                                  
            vals = {}                                                               
            amt_needed = 0
            multiple = 0
            sum = 0
            amt_remaining = 0
            total_amt_to_move = -1
            product = self.pool.get('product.product').browse(cr, uid, prod_id, context=context)
            vals.update({'product_name': product.id, 'state': 'draft'})
            if product.default_code:
                vals['reference'] = product.default_code
            else:
                vals['reference'] = ''
            for multi_loc in product.multi_location_ids:                                                 
                if not multi_loc.location_active:
                    continue
                if multi_loc.storage_type == 'primary' and multi_loc.location_active:                     
                    vals['primary_loc'] = multi_loc.location_name.id                       
                    if multi_loc.qty_available < multi_loc.min_level:
                           amt_needed = multi_loc.target_level-multi_loc.qty_available
                           if amt_needed > 0:
                               try:
                                   multiple = amt_needed / multi_loc.qty_multiple
                               except:
                                   multiple = 0
                               total_amt_to_move = multi_loc.qty_multiple * int(multiple)
                               amt_remaining = total_amt_to_move
                           else:
                               break
                    else:
                        break
                    
            for multi_loc in product.multi_location_ids:
                if not multi_loc.location_active:
                    continue

                if multi_loc.storage_type == 'primary' and multi_loc.location_active:
                    vals['primary_loc'] = multi_loc.location_name.id
                if multi_loc.storage_type != 'primary' and multi_loc.location_active and multi_loc.qty_available > 0 and amt_remaining > 0:
                    vals['replenish_loc'] = multi_loc.location_name.id
                    if multi_loc.qty_available >= amt_remaining:
                        vals.update({'amt_left': multi_loc.qty_available - amt_remaining, 'amt_to_move': amt_remaining})
                        amt_remaining = 0
                    else:
                        vals.update({'amt_left': 0, 'amt_to_move': multi_loc.qty_available})
                        amt_remaining = amt_remaining-multi_loc.qty_available

                if not 'replenish_loc' in vals or not 'primary_loc' in vals or vals['amt_to_move'] <= 0:
                #if there is no replenish_loc/primary_loc/or amount to move continue with next multi location line
                    continue
                vals['date_moved'] = time.strftime('%m-%d-%Y %H:%M:%S')
                id_list.append(replenish_obj.create(cr, uid, vals, context=context))
                del vals['amt_left']
                del vals['amt_to_move']
                del vals['replenish_loc']
        id_list = list(set(id_list))
        return {
            'domain': [('id', 'in', id_list)],
            'name': _('Replenish Data'),
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'multilocation.replenish',
            'views': False,
            'type': 'ir.actions.act_window',
            'target': 'current',
            }
        
replenish_update()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: